/******************************************************************************/
//
// Name: 	Interfacing ADSP-21365 with AD7865 on PDAP
//
/******************************************************************************

(C) Copyright 2004 - Analog Devices, Inc.  All rights reserved.

File Name:		ADSP-21365_with_AD7865_on_PDAP.asm

Date Modified:	02/12/04				Rev 1.0

Software:       VisualDSP++3.5 (November update)

Purpose:		To receive data from AD7865
				
*******************************************************************************/

#include <def21365.h>
#include <SRU.h>

#define ITERATIONS	25

.section/pm seg_rth;
	nop;nop;nop;nop;
  	nop;jump start;nop;nop;
  	
.section/pm seg_dai;
	jump isr;
	rti;
	rti;
	rti;
 	
.section/dm seg_dmda;
	.var dest[2];
	.var adc_data[4*ITERATIONS];
	.var counter=0;

.section/pm seg_pmco;
start:
	// Enabling interrupts
	bit set imask P0I;
	
	// Using FLAG0 for triggering on LA
	bit set FLAGS FLG0O;
	bit clr FLAGS FLG0;
	
	ustat1 = dm(DAI_IRPTL_PRI);
	bit set ustat1 IDP_DMA0_INT;
	dm(DAI_IRPTL_PRI) = ustat1;
	
	ustat1 = dm(DAI_IRPTL_RE);
	bit set ustat1 IDP_DMA0_INT;
	dm(DAI_IRPTL_RE) = ustat1;
	
	bit set mode1 IRPTEN;
	
	// First clear the IDP FIFO
	// This is achieved by setting and clearing the IDP_ENABLE bit
	ustat1 = dm(IDP_CTL0);
	bit set ustat1 IDP_EN;
	dm(IDP_CTL0) = ustat1;
	
	bit clr ustat1 IDP_EN;
	dm(IDP_CTL0) = ustat1;
	
	// Set up DMA parameter registers
	r0 = dest;
	dm(IDP_DMA_I0) = r0;
	
	r0 = 1;
	dm(IDP_DMA_M0) = r0;
	
	r0 = @dest;
	dm(IDP_DMA_C0) = r0;
	
	// CLK and frame sync inputs should be connected to low
	SRU(LOW,IDP0_FS_I);
	SRU(LOW,IDP0_CLK_I);
	
	// Setting the PDAP control register
	// Selecting PDAP instead of IDP0
	// Packing mode set to 16 to 32 packing.
	// Masking off data on 19th and 20th DAI pins.
	ustat1 = 0x0;
	dm(IDP_PP_CTL)=ustat1;
	
	bit set ustat1 IDP_PDAP_PACKING2|IDP_PDAPMASK|IDP_PDAP_RESET;
	bit clr ustat1 IDP_P19_PDAPMASK|IDP_P20_PDAPMASK;
	dm(IDP_PP_CTL)=ustat1;
	
	// SPORT0 FS signal will be available on DAI pin 20
	SRU(SPORT0_FS_O,DAI_PB20_I);
	SRU(HIGH,PBEN20_I);
	
	// Programming SPORT registers
	r0 = 0x0;
	dm(SPCTL0) = r0;
		
	r0 = 0x20001f;
	dm(DIV0) = r0;
	
	// Dummy write to initiate TFS
	r0 = 0xaaaaaaaa;
	dm(TXSP0A) = r0;
	
	// Programming SPORT control register.
	// FS of SPORT will be used as CONVST signal for AD7865
	r0 = SPEN_A | SLEN32 | ICLK | IFS | FSR | DITFS | LFS | SPTRAN;
	dm(SPCTL0) = r0;
	
	// DAI pin 2 is used as PDAP_CLK signal
	SRU(DAI_PB02_O,IDP0_CLK_I);
	SRU(LOW,PBEN02_I);
	
	// Enable IDP DMA enable bit
	r0 = IDP_DMA_EN;
	dm(IDP_CTL0) = r0;
		
	// Enable IDP DMA enable bit for channel 0
	r0 = IDP_DMA_EN0;
	dm(IDP_CTL1) = r0;
	
	// Enable PDAP DMA
	ustat1 = dm(IDP_PP_CTL);
	bit set ustat1 IDP_PDAP_EN;
	dm(IDP_PP_CTL)=ustat1;
	
	// Enable IDP enable bit for channel 0
	ustat1 = dm(IDP_CTL1);
	bit set ustat1 IDP_EN0;
	dm(IDP_CTL1) = ustat1;
	
	// Enable IDP enable bit
	ustat1 = dm(IDP_CTL0);
	bit set ustat1 IDP_EN;
	dm(IDP_CTL0) = ustat1;
	
	// Initializing DAG registers
	i0 = adc_data;
	m0 = 1;
	
	// Register used to halt the program after some iterations
	r15 = ITERATIONS;
	
	nop;
	nop;
	jump(pc,0);

// PDAP interrupt service routine
.section/pm seg_pmco;
isr:
	// Disable IDP DMA enable bit
	ustat1 = dm(IDP_CTL0);
	bit clr ustat1 IDP_DMA_EN;
	dm(IDP_CTL0) = ustat1;
	
	ustat1 = dm(DAI_IRPTL_H);
	
	// Rearranging the data
	r1 = 0x3fff;
	r2 = 0x3fff0000;
	r3 = dm(dest);
	r0 = r3 and r1;
	dm(i0,m0) = r0;
	
	r0 = r3 and r2;
	r0 = lshift r0 by -16;
	r0 = r0 and r1;
	dm(i0,m0) = r0;
	
	r3 = dm(dest+1);
	r0 = r3 and r1;
	dm(i0,m0) = r0;
	
	r0 = r3 and r2;
	r0 = lshift r0 by -16;
	r0 = r0 and r1;
	dm(i0,m0) = r0;
	
	// Increment counter
	r0 = dm(counter);
	r0 = r0 + 1;
	dm(counter) = r0;
	
	comp(r0,r15);
	if eq jump done;
	
	// Clear the destination data
	i1 = dest;
	m1 = 1;
	
	r0 = 0x0;
	lcntr = 2, do clear until lce;
clear:	dm(i1, m1) = r0;
		
	// Reprogram DMA parameter registers
	r0 = dest;
	dm(IDP_DMA_I0) = r0;
	
	r0 = 1;
	dm(IDP_DMA_M0) = r0;
	
	r0 = @dest;
	dm(IDP_DMA_C0) = r0;
	
	// Re-enable IDP DMA enable bit
	ustat1 = dm(IDP_CTL0);
	bit set ustat1 IDP_DMA_EN;
	dm(IDP_CTL0) = ustat1;
	
	rti;
	
done:
	// Setting FLAG 0 to capture signals on LA
	bit set FLAGS FLG0;	
	nop;
	nop;
	jump(pc,0);
